@@ -25,16 +25,53 @@ use {
     solana_sdk::{
         clock::{Slot, UnixTimestamp},
         commitment_config::CommitmentConfig,
-        deserialize_utils::default_on_eof,
         instruction::CompiledInstruction,
-        message::{Message, MessageHeader},
-        pubkey::Pubkey,
+        message::{v0::LoadedAddresses, AccountKeys, Message, MessageHeader},
         sanitize::Sanitize,
         signature::Signature,
-        transaction::{Result, Transaction, TransactionError},
+        transaction::{Result, Transaction, TransactionError, VersionedTransaction},
     },
     std::fmt,
 };
+
+/// Represents types that can be encoded into one of several encoding formats
+pub trait Encodable {
+    type Encoded;
+    fn encode(self, encoding: UiTransactionEncoding) -> Self::Encoded;
+}
+
+#[derive(Serialize, Deserialize, Clone, Copy, Debug, Eq, Hash, PartialEq)]
+#[serde(rename_all = "camelCase")]
+pub enum UiTransactionEncoding {
+    Binary, // Legacy. Retained for RPC backwards compatibility
+    Base64,
+    Base58,
+    Json,
+    JsonParsed,
+}
+
+impl fmt::Display for UiTransactionEncoding {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let v = serde_json::to_value(self).map_err(|_| fmt::Error)?;
+        let s = v.as_str().ok_or(fmt::Error)?;
+        write!(f, "{}", s)
+    }
+}
+
+#[derive(Debug, Clone, Copy, Eq, Hash, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub enum TransactionDetails {
+    Full,
+    Signatures,
+    None,
+}
+
+impl Default for TransactionDetails {
+    fn default() -> Self {
+        Self::Full
+    }
+}
+
 /// A duplicate representation of an Instruction for pretty JSON serialization
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase", untagged)]
@@ -43,6 +80,19 @@ pub enum UiInstruction {
     Parsed(UiParsedInstruction),
 }
 
+impl UiInstruction {
+    fn parse(instruction: &CompiledInstruction, account_keys: &AccountKeys) -> Self {
+        let program_id = &account_keys[instruction.program_id_index as usize];
+        if let Ok(parsed_instruction) = parse(program_id, instruction, account_keys) {
+            UiInstruction::Parsed(UiParsedInstruction::Parsed(parsed_instruction))
+        } else {
+            UiInstruction::Parsed(UiParsedInstruction::PartiallyDecoded(
+                UiPartiallyDecodedInstruction::from(instruction, account_keys),
+            ))
+        }
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase", untagged)]
 pub enum UiParsedInstruction {
@@ -50,19 +100,6 @@ pub enum UiParsedInstruction {
     PartiallyDecoded(UiPartiallyDecodedInstruction),
 }
 
-impl UiInstruction {
-    fn parse(instruction: &CompiledInstruction, message: &Message) -> Self {
-        let program_id = instruction.program_id(&message.account_keys);
-        if let Ok(parsed_instruction) = parse(program_id, instruction, &message.account_keys) {
-            UiInstruction::Parsed(UiParsedInstruction::Parsed(parsed_instruction))
-        } else {
-            UiInstruction::Parsed(UiParsedInstruction::PartiallyDecoded(
-                UiPartiallyDecodedInstruction::from(instruction, &message.account_keys),
-            ))
-        }
-    }
-}
-
 /// A duplicate representation of a CompiledInstruction for pretty JSON serialization
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
@@ -92,7 +129,7 @@ pub struct UiPartiallyDecodedInstruction {
 }
 
 impl UiPartiallyDecodedInstruction {
-    fn from(instruction: &CompiledInstruction, account_keys: &[Pubkey]) -> Self {
+    fn from(instruction: &CompiledInstruction, account_keys: &AccountKeys) -> Self {
         Self {
             program_id: account_keys[instruction.program_id_index as usize].to_string(),
             accounts: instruction
@@ -122,7 +159,34 @@ pub struct UiInnerInstructions {
     pub instructions: Vec<UiInstruction>,
 }
 
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+impl UiInnerInstructions {
+    fn parse(inner_instructions: InnerInstructions, message: &Message) -> Self {
+        let account_keys = AccountKeys::new(&message.account_keys, None);
+        Self {
+            index: inner_instructions.index,
+            instructions: inner_instructions
+                .instructions
+                .iter()
+                .map(|ix| UiInstruction::parse(ix, &account_keys))
+                .collect(),
+        }
+    }
+}
+
+impl From<InnerInstructions> for UiInnerInstructions {
+    fn from(inner_instructions: InnerInstructions) -> Self {
+        Self {
+            index: inner_instructions.index,
+            instructions: inner_instructions
+                .instructions
+                .iter()
+                .map(|ix| UiInstruction::Compiled(ix.into()))
+                .collect(),
+        }
+    }
+}
+
+#[derive(Clone, Debug, PartialEq)]
 pub struct TransactionTokenBalance {
     pub account_index: u8,
     pub mint: String,
@@ -155,49 +219,18 @@ impl From<TransactionTokenBalance> for UiTransactionTokenBalance {
     }
 }
 
-impl UiInnerInstructions {
-    fn parse(inner_instructions: InnerInstructions, message: &Message) -> Self {
-        Self {
-            index: inner_instructions.index,
-            instructions: inner_instructions
-                .instructions
-                .iter()
-                .map(|ix| UiInstruction::parse(ix, message))
-                .collect(),
-        }
-    }
-}
-
-impl From<InnerInstructions> for UiInnerInstructions {
-    fn from(inner_instructions: InnerInstructions) -> Self {
-        Self {
-            index: inner_instructions.index,
-            instructions: inner_instructions
-                .instructions
-                .iter()
-                .map(|ix| UiInstruction::Compiled(ix.into()))
-                .collect(),
-        }
-    }
-}
-
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(Clone, Debug, PartialEq)]
 pub struct TransactionStatusMeta {
     pub status: Result<()>,
     pub fee: u64,
     pub pre_balances: Vec<u64>,
     pub post_balances: Vec<u64>,
-    #[serde(deserialize_with = "default_on_eof")]
     pub inner_instructions: Option<Vec<InnerInstructions>>,
-    #[serde(deserialize_with = "default_on_eof")]
     pub log_messages: Option<Vec<String>>,
-    #[serde(deserialize_with = "default_on_eof")]
     pub pre_token_balances: Option<Vec<TransactionTokenBalance>>,
-    #[serde(deserialize_with = "default_on_eof")]
     pub post_token_balances: Option<Vec<TransactionTokenBalance>>,
-    #[serde(deserialize_with = "default_on_eof")]
     pub rewards: Option<Rewards>,
+    pub loaded_addresses: LoadedAddresses,
 }
 
 impl Default for TransactionStatusMeta {
@@ -212,6 +245,7 @@ impl Default for TransactionStatusMeta {
             pre_token_balances: None,
             post_token_balances: None,
             rewards: None,
+            loaded_addresses: LoadedAddresses::default(),
         }
     }
 }
@@ -248,10 +282,10 @@ impl UiTransactionStatusMeta {
             log_messages: meta.log_messages,
             pre_token_balances: meta
                 .pre_token_balances
-                .map(|balance| balance.into_iter().map(|balance| balance.into()).collect()),
+                .map(|balance| balance.into_iter().map(Into::into).collect()),
             post_token_balances: meta
                 .post_token_balances
-                .map(|balance| balance.into_iter().map(|balance| balance.into()).collect()),
+                .map(|balance| balance.into_iter().map(Into::into).collect()),
             rewards: meta.rewards,
         }
     }
@@ -267,14 +301,14 @@ impl From<TransactionStatusMeta> for UiTransactionStatusMeta {
             post_balances: meta.post_balances,
             inner_instructions: meta
                 .inner_instructions
-                .map(|ixs| ixs.into_iter().map(|ix| ix.into()).collect()),
+                .map(|ixs| ixs.into_iter().map(Into::into).collect()),
             log_messages: meta.log_messages,
             pre_token_balances: meta
                 .pre_token_balances
-                .map(|balance| balance.into_iter().map(|balance| balance.into()).collect()),
+                .map(|balance| balance.into_iter().map(Into::into).collect()),
             post_token_balances: meta
                 .post_token_balances
-                .map(|balance| balance.into_iter().map(|balance| balance.into()).collect()),
+                .map(|balance| balance.into_iter().map(Into::into).collect()),
             rewards: meta.rewards,
         }
     }
@@ -333,8 +367,7 @@ impl TransactionStatus {
     }
 }
 
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(Clone, Debug, PartialEq)]
 pub struct ConfirmedTransactionStatusWithSignature {
     pub signature: Signature,
     pub slot: Slot,
@@ -360,30 +393,68 @@ pub struct ConfirmedBlock {
     pub previous_blockhash: String,
     pub blockhash: String,
     pub parent_slot: Slot,
-    pub transactions: Vec<TransactionWithMetadata>,
+    pub transactions: Vec<TransactionWithStatusMeta>,
     pub rewards: Rewards,
     pub block_time: Option<UnixTimestamp>,
     pub block_height: Option<u64>,
 }
 
+// Confirmed block with type guarantees that transaction metadata
+// is always present. Used for uploading to BigTable.
 #[derive(Clone, Debug, PartialEq)]
-pub struct ConfirmedBlockWithOptionalMetadata {
+pub struct VersionedConfirmedBlock {
     pub previous_blockhash: String,
     pub blockhash: String,
     pub parent_slot: Slot,
-    pub transactions: Vec<TransactionWithOptionalMetadata>,
+    pub transactions: Vec<VersionedTransactionWithStatusMeta>,
     pub rewards: Rewards,
     pub block_time: Option<UnixTimestamp>,
     pub block_height: Option<u64>,
 }
 
-impl From<ConfirmedBlock> for ConfirmedBlockWithOptionalMetadata {
-    fn from(block: ConfirmedBlock) -> Self {
+// Confirmed block which only supports legacy transactions. Used
+// until migration to versioned transactions is completed.
+pub struct LegacyConfirmedBlock {
+    pub previous_blockhash: String,
+    pub blockhash: String,
+    pub parent_slot: Slot,
+    pub transactions: Vec<LegacyTransactionWithStatusMeta>,
+    pub rewards: Rewards,
+    pub block_time: Option<UnixTimestamp>,
+    pub block_height: Option<u64>,
+}
+
+impl ConfirmedBlock {
+    /// Downgrades a versioned block into a legacy block type
+    /// if it only contains legacy transactions
+    pub fn into_legacy_block(self) -> Option<LegacyConfirmedBlock> {
+        Some(LegacyConfirmedBlock {
+            previous_blockhash: self.previous_blockhash,
+            blockhash: self.blockhash,
+            parent_slot: self.parent_slot,
+            transactions: self
+                .transactions
+                .into_iter()
+                .map(|tx_with_meta| tx_with_meta.into_legacy_transaction_with_meta())
+                .collect::<Option<Vec<_>>>()?,
+            rewards: self.rewards,
+            block_time: self.block_time,
+            block_height: self.block_height,
+        })
+    }
+}
+
+impl From<VersionedConfirmedBlock> for ConfirmedBlock {
+    fn from(block: VersionedConfirmedBlock) -> Self {
         Self {
             previous_blockhash: block.previous_blockhash,
             blockhash: block.blockhash,
             parent_slot: block.parent_slot,
-            transactions: block.transactions.into_iter().map(Into::into).collect(),
+            transactions: block
+                .transactions
+                .into_iter()
+                .map(TransactionWithStatusMeta::Complete)
+                .collect(),
             rewards: block.rewards,
             block_time: block.block_time,
             block_height: block.block_height,
@@ -391,9 +462,10 @@ impl From<ConfirmedBlock> for ConfirmedBlockWithOptionalMetadata {
     }
 }
 
-impl ConfirmedBlockWithOptionalMetadata {
-    pub fn encode(self, encoding: UiTransactionEncoding) -> EncodedConfirmedBlock {
-        EncodedConfirmedBlock {
+impl Encodable for LegacyConfirmedBlock {
+    type Encoded = EncodedConfirmedBlock;
+    fn encode(self, encoding: UiTransactionEncoding) -> Self::Encoded {
+        Self::Encoded {
             previous_blockhash: self.previous_blockhash,
             blockhash: self.blockhash,
             parent_slot: self.parent_slot,
@@ -407,7 +479,9 @@ impl ConfirmedBlockWithOptionalMetadata {
             block_height: self.block_height,
         }
     }
+}
 
+impl LegacyConfirmedBlock {
     pub fn configure(
         self,
         encoding: UiTransactionEncoding,
@@ -419,7 +493,7 @@ impl ConfirmedBlockWithOptionalMetadata {
                 Some(
                     self.transactions
                         .into_iter()
-                        .map(|tx| tx.encode(encoding))
+                        .map(|tx_with_meta| tx_with_meta.encode(encoding))
                         .collect(),
                 ),
                 None,
@@ -464,6 +538,20 @@ pub struct EncodedConfirmedBlock {
     pub block_height: Option<u64>,
 }
 
+impl From<UiConfirmedBlock> for EncodedConfirmedBlock {
+    fn from(block: UiConfirmedBlock) -> Self {
+        Self {
+            previous_blockhash: block.previous_blockhash,
+            blockhash: block.blockhash,
+            parent_slot: block.parent_slot,
+            transactions: block.transactions.unwrap_or_default(),
+            rewards: block.rewards.unwrap_or_default(),
+            block_time: block.block_time,
+            block_height: block.block_height,
+        }
+    }
+}
+
 #[derive(Debug, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct UiConfirmedBlock {
@@ -495,139 +583,78 @@ impl From<EncodedConfirmedBlock> for UiConfirmedBlock {
     }
 }
 
-impl From<UiConfirmedBlock> for EncodedConfirmedBlock {
-    fn from(block: UiConfirmedBlock) -> Self {
-        Self {
-            previous_blockhash: block.previous_blockhash,
-            blockhash: block.blockhash,
-            parent_slot: block.parent_slot,
-            transactions: block.transactions.unwrap_or_default(),
-            rewards: block.rewards.unwrap_or_default(),
-            block_time: block.block_time,
-            block_height: block.block_height,
-        }
-    }
-}
-
-#[derive(Debug, Clone, Copy, Eq, Hash, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub enum TransactionDetails {
-    Full,
-    Signatures,
-    None,
-}
-
-impl Default for TransactionDetails {
-    fn default() -> Self {
-        Self::Full
-    }
-}
-
-#[derive(Debug, Clone, PartialEq)]
-pub struct ConfirmedTransaction {
-    pub slot: Slot,
-    pub transaction: TransactionWithMetadata,
-    pub block_time: Option<UnixTimestamp>,
-}
-
-#[derive(Debug, Clone, PartialEq)]
-pub struct ConfirmedTransactionWithOptionalMetadata {
-    pub slot: Slot,
-    pub transaction: TransactionWithOptionalMetadata,
-    pub block_time: Option<UnixTimestamp>,
-}
-
-impl From<ConfirmedTransaction> for ConfirmedTransactionWithOptionalMetadata {
-    fn from(confirmed_tx: ConfirmedTransaction) -> Self {
-        Self {
-            slot: confirmed_tx.slot,
-            transaction: confirmed_tx.transaction.into(),
-            block_time: confirmed_tx.block_time,
-        }
-    }
-}
-
-impl ConfirmedTransactionWithOptionalMetadata {
-    pub fn encode(self, encoding: UiTransactionEncoding) -> EncodedConfirmedTransaction {
-        EncodedConfirmedTransaction {
-            slot: self.slot,
-            transaction: self.transaction.encode(encoding),
-            block_time: self.block_time,
-        }
-    }
-}
-
-#[derive(Debug, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub struct EncodedConfirmedTransaction {
-    pub slot: Slot,
-    #[serde(flatten)]
-    pub transaction: EncodedTransactionWithStatusMeta,
-    pub block_time: Option<UnixTimestamp>,
-}
-
-/// A duplicate representation of a Transaction for pretty JSON serialization
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub struct UiTransaction {
-    pub signatures: Vec<String>,
-    pub message: UiMessage,
-}
-
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase", untagged)]
-pub enum UiMessage {
-    Parsed(UiParsedMessage),
-    Raw(UiRawMessage),
-}
-
-/// A duplicate representation of a Message, in raw format, for pretty JSON serialization
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub struct UiRawMessage {
-    pub header: MessageHeader,
-    pub account_keys: Vec<String>,
-    pub recent_blockhash: String,
-    pub instructions: Vec<UiCompiledInstruction>,
-}
-
-/// A duplicate representation of a Message, in parsed format, for pretty JSON serialization
-#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub struct UiParsedMessage {
-    pub account_keys: Vec<ParsedAccount>,
-    pub recent_blockhash: String,
-    pub instructions: Vec<UiInstruction>,
+#[derive(Clone, Debug, PartialEq)]
+#[allow(clippy::large_enum_variant)]
+pub enum TransactionWithStatusMeta {
+    // Very old transactions may be missing metadata
+    MissingMetadata(Transaction),
+    // Versioned stored transaction always have metadata
+    Complete(VersionedTransactionWithStatusMeta),
 }
 
 #[derive(Clone, Debug, PartialEq)]
-pub struct TransactionWithMetadata {
-    pub transaction: Transaction,
+pub struct VersionedTransactionWithStatusMeta {
+    pub transaction: VersionedTransaction,
     pub meta: TransactionStatusMeta,
 }
 
-#[derive(Clone, Debug, PartialEq)]
-pub struct TransactionWithOptionalMetadata {
+pub struct LegacyTransactionWithStatusMeta {
     pub transaction: Transaction,
     pub meta: Option<TransactionStatusMeta>,
 }
 
-impl From<TransactionWithMetadata> for TransactionWithOptionalMetadata {
-    fn from(tx_with_meta: TransactionWithMetadata) -> Self {
-        Self {
-            transaction: tx_with_meta.transaction,
-            meta: Some(tx_with_meta.meta),
+impl TransactionWithStatusMeta {
+    pub fn transaction_signature(&self) -> &Signature {
+        match self {
+            Self::MissingMetadata(transaction) => &transaction.signatures[0],
+            Self::Complete(VersionedTransactionWithStatusMeta { transaction, .. }) => {
+                &transaction.signatures[0]
+            }
+        }
+    }
+
+    pub fn into_legacy_transaction_with_meta(self) -> Option<LegacyTransactionWithStatusMeta> {
+        match self {
+            TransactionWithStatusMeta::MissingMetadata(transaction) => {
+                Some(LegacyTransactionWithStatusMeta {
+                    transaction,
+                    meta: None,
+                })
+            }
+            TransactionWithStatusMeta::Complete(tx_with_meta) => {
+                tx_with_meta.into_legacy_transaction_with_meta()
+            }
         }
     }
 }
 
-impl TransactionWithOptionalMetadata {
-    fn encode(self, encoding: UiTransactionEncoding) -> EncodedTransactionWithStatusMeta {
-        let message = self.transaction.message();
-        let meta = self.meta.map(|meta| meta.encode(encoding, message));
-        EncodedTransactionWithStatusMeta {
-            transaction: EncodedTransaction::encode(self.transaction, encoding),
-            meta,
+impl VersionedTransactionWithStatusMeta {
+    pub fn account_keys(&self) -> AccountKeys {
+        AccountKeys::new(
+            self.transaction.message.static_account_keys(),
+            Some(&self.meta.loaded_addresses),
+        )
+    }
+
+    pub fn into_legacy_transaction_with_meta(self) -> Option<LegacyTransactionWithStatusMeta> {
+        Some(LegacyTransactionWithStatusMeta {
+            transaction: self.transaction.into_legacy_transaction()?,
+            meta: Some(self.meta),
+        })
+    }
+}
+
+impl Encodable for LegacyTransactionWithStatusMeta {
+    type Encoded = EncodedTransactionWithStatusMeta;
+    fn encode(self, encoding: UiTransactionEncoding) -> Self::Encoded {
+        Self::Encoded {
+            transaction: self.transaction.encode(encoding),
+            meta: self.meta.map(|meta| match encoding {
+                UiTransactionEncoding::JsonParsed => {
+                    UiTransactionStatusMeta::parse(meta, &self.transaction.message)
+                }
+                _ => UiTransactionStatusMeta::from(meta),
+            }),
         }
     }
 }
@@ -639,31 +666,58 @@ pub struct EncodedTransactionWithStatusMeta {
     pub meta: Option<UiTransactionStatusMeta>,
 }
 
-impl TransactionStatusMeta {
-    fn encode(self, encoding: UiTransactionEncoding, message: &Message) -> UiTransactionStatusMeta {
-        match encoding {
-            UiTransactionEncoding::JsonParsed => UiTransactionStatusMeta::parse(self, message),
-            _ => self.into(),
+#[derive(Debug, Clone, PartialEq)]
+pub struct ConfirmedTransactionWithStatusMeta {
+    pub slot: Slot,
+    pub tx_with_meta: TransactionWithStatusMeta,
+    pub block_time: Option<UnixTimestamp>,
+}
+
+#[derive(Debug, Clone, PartialEq)]
+pub struct VersionedConfirmedTransactionWithStatusMeta {
+    pub slot: Slot,
+    pub tx_with_meta: VersionedTransactionWithStatusMeta,
+    pub block_time: Option<UnixTimestamp>,
+}
+
+pub struct LegacyConfirmedTransactionWithStatusMeta {
+    pub slot: Slot,
+    pub tx_with_meta: LegacyTransactionWithStatusMeta,
+    pub block_time: Option<UnixTimestamp>,
+}
+
+impl Encodable for LegacyConfirmedTransactionWithStatusMeta {
+    type Encoded = EncodedConfirmedTransactionWithStatusMeta;
+    fn encode(self, encoding: UiTransactionEncoding) -> Self::Encoded {
+        Self::Encoded {
+            slot: self.slot,
+            transaction: self.tx_with_meta.encode(encoding),
+            block_time: self.block_time,
         }
     }
 }
 
-#[derive(Serialize, Deserialize, Clone, Copy, Debug, Eq, Hash, PartialEq)]
-#[serde(rename_all = "camelCase")]
-pub enum UiTransactionEncoding {
-    Binary, // Legacy. Retained for RPC backwards compatibility
-    Base64,
-    Base58,
-    Json,
-    JsonParsed,
+impl ConfirmedTransactionWithStatusMeta {
+    /// Downgrades a versioned confirmed transaction into a legacy
+    /// confirmed transaction if it contains a legacy transaction.
+    pub fn into_legacy_confirmed_transaction(
+        self,
+    ) -> Option<LegacyConfirmedTransactionWithStatusMeta> {
+        Some(LegacyConfirmedTransactionWithStatusMeta {
+            tx_with_meta: self.tx_with_meta.into_legacy_transaction_with_meta()?,
+            block_time: self.block_time,
+            slot: self.slot,
+        })
+    }
 }
 
-impl fmt::Display for UiTransactionEncoding {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let v = serde_json::to_value(self).map_err(|_| fmt::Error)?;
-        let s = v.as_str().ok_or(fmt::Error)?;
-        write!(f, "{}", s)
-    }
+#[derive(Debug, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct EncodedConfirmedTransactionWithStatusMeta {
+    pub slot: Slot,
+    #[serde(flatten)]
+    pub transaction: EncodedTransactionWithStatusMeta,
+    pub block_time: Option<UnixTimestamp>,
 }
 
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
@@ -674,63 +728,32 @@ pub enum EncodedTransaction {
     Json(UiTransaction),
 }
 
-impl EncodedTransaction {
-    pub fn encode(transaction: Transaction, encoding: UiTransactionEncoding) -> Self {
+impl Encodable for &Transaction {
+    type Encoded = EncodedTransaction;
+    fn encode(self, encoding: UiTransactionEncoding) -> Self::Encoded {
         match encoding {
             UiTransactionEncoding::Binary => EncodedTransaction::LegacyBinary(
-                bs58::encode(bincode::serialize(&transaction).unwrap()).into_string(),
+                bs58::encode(bincode::serialize(self).unwrap()).into_string(),
             ),
             UiTransactionEncoding::Base58 => EncodedTransaction::Binary(
-                bs58::encode(bincode::serialize(&transaction).unwrap()).into_string(),
+                bs58::encode(bincode::serialize(self).unwrap()).into_string(),
                 encoding,
             ),
             UiTransactionEncoding::Base64 => EncodedTransaction::Binary(
-                base64::encode(bincode::serialize(&transaction).unwrap()),
+                base64::encode(bincode::serialize(self).unwrap()),
                 encoding,
             ),
             UiTransactionEncoding::Json | UiTransactionEncoding::JsonParsed => {
-                let message = if encoding == UiTransactionEncoding::Json {
-                    UiMessage::Raw(UiRawMessage {
-                        header: transaction.message.header,
-                        account_keys: transaction
-                            .message
-                            .account_keys
-                            .iter()
-                            .map(|pubkey| pubkey.to_string())
-                            .collect(),
-                        recent_blockhash: transaction.message.recent_blockhash.to_string(),
-                        instructions: transaction
-                            .message
-                            .instructions
-                            .iter()
-                            .map(|instruction| instruction.into())
-                            .collect(),
-                    })
-                } else {
-                    UiMessage::Parsed(UiParsedMessage {
-                        account_keys: parse_accounts(&transaction.message),
-                        recent_blockhash: transaction.message.recent_blockhash.to_string(),
-                        instructions: transaction
-                            .message
-                            .instructions
-                            .iter()
-                            .map(|instruction| {
-                                UiInstruction::parse(instruction, &transaction.message)
-                            })
-                            .collect(),
-                    })
-                };
                 EncodedTransaction::Json(UiTransaction {
-                    signatures: transaction
-                        .signatures
-                        .iter()
-                        .map(|sig| sig.to_string())
-                        .collect(),
-                    message,
+                    signatures: self.signatures.iter().map(ToString::to_string).collect(),
+                    message: self.message.encode(encoding),
                 })
             }
         }
     }
+}
+
+impl EncodedTransaction {
     pub fn decode(&self) -> Option<Transaction> {
         let transaction: Option<Transaction> = match self {
             EncodedTransaction::Json(_) => None,
@@ -755,6 +778,65 @@ impl EncodedTransaction {
     }
 }
 
+/// A duplicate representation of a Transaction for pretty JSON serialization
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct UiTransaction {
+    pub signatures: Vec<String>,
+    pub message: UiMessage,
+}
+
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase", untagged)]
+pub enum UiMessage {
+    Parsed(UiParsedMessage),
+    Raw(UiRawMessage),
+}
+
+impl Encodable for &Message {
+    type Encoded = UiMessage;
+    fn encode(self, encoding: UiTransactionEncoding) -> Self::Encoded {
+        if encoding == UiTransactionEncoding::JsonParsed {
+            let account_keys = AccountKeys::new(&self.account_keys, None);
+            UiMessage::Parsed(UiParsedMessage {
+                account_keys: parse_accounts(self),
+                recent_blockhash: self.recent_blockhash.to_string(),
+                instructions: self
+                    .instructions
+                    .iter()
+                    .map(|instruction| UiInstruction::parse(instruction, &account_keys))
+                    .collect(),
+            })
+        } else {
+            UiMessage::Raw(UiRawMessage {
+                header: self.header,
+                account_keys: self.account_keys.iter().map(ToString::to_string).collect(),
+                recent_blockhash: self.recent_blockhash.to_string(),
+                instructions: self.instructions.iter().map(Into::into).collect(),
+            })
+        }
+    }
+}
+
+/// A duplicate representation of a Message, in raw format, for pretty JSON serialization
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct UiRawMessage {
+    pub header: MessageHeader,
+    pub account_keys: Vec<String>,
+    pub recent_blockhash: String,
+    pub instructions: Vec<UiCompiledInstruction>,
+}
+
+/// A duplicate representation of a Message, in parsed format, for pretty JSON serialization
+#[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct UiParsedMessage {
+    pub account_keys: Vec<ParsedAccount>,
+    pub recent_blockhash: String,
+    pub instructions: Vec<UiInstruction>,
+}
+
 // A serialized `Vec<TransactionByAddrInfo>` is stored in the `tx-by-addr` table.  The row keys are
 // the one's compliment of the slot so that rows may be listed in reverse order
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
