@@ -3,18 +3,18 @@ use {
         check_num_accounts, ParsableProgram, ParseInstructionError, ParsedInstructionEnum,
     },
     serde_json::json,
-    solana_sdk::{instruction::CompiledInstruction, pubkey::Pubkey},
+    solana_sdk::{instruction::CompiledInstruction, message::AccountKeys, pubkey::Pubkey},
 };
 
-// A helper function to convert spl_associated_token_account_v1_0::id() as spl_sdk::pubkey::Pubkey
+// A helper function to convert spl_associated_token_account::id() as spl_sdk::pubkey::Pubkey
 // to solana_sdk::pubkey::Pubkey
-pub fn spl_associated_token_id_v1_0() -> Pubkey {
-    Pubkey::new_from_array(spl_associated_token_account_v1_0::id().to_bytes())
+pub fn spl_associated_token_id() -> Pubkey {
+    Pubkey::new_from_array(spl_associated_token_account::id().to_bytes())
 }
 
 pub fn parse_associated_token(
     instruction: &CompiledInstruction,
-    account_keys: &[Pubkey],
+    account_keys: &AccountKeys,
 ) -> Result<ParsedInstructionEnum, ParseInstructionError> {
     match instruction.accounts.iter().max() {
         Some(index) if (*index as usize) < account_keys.len() => {}
@@ -51,7 +51,7 @@ fn check_num_associated_token_accounts(
 mod test {
     use {
         super::*,
-        spl_associated_token_account_v1_0::{
+        spl_associated_token_account::{
             create_associated_token_account,
             solana_program::{
                 instruction::CompiledInstruction as SplAssociatedTokenCompiledInstruction,
@@ -88,8 +88,9 @@ mod test {
         );
         let message = Message::new(&[create_ix], None);
         let compiled_instruction = convert_compiled_instruction(&message.instructions[0]);
+        let account_keys = AccountKeys::new(&keys, None);
         assert_eq!(
-            parse_associated_token(&compiled_instruction, &keys).unwrap(),
+            parse_associated_token(&compiled_instruction, &account_keys).unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "create".to_string(),
                 info: json!({
