@@ -6,13 +6,13 @@ use {
     serde_json::json,
     solana_sdk::{
         instruction::CompiledInstruction, loader_instruction::LoaderInstruction,
-        loader_upgradeable_instruction::UpgradeableLoaderInstruction, pubkey::Pubkey,
+        loader_upgradeable_instruction::UpgradeableLoaderInstruction, message::AccountKeys,
     },
 };
 
 pub fn parse_bpf_loader(
     instruction: &CompiledInstruction,
-    account_keys: &[Pubkey],
+    account_keys: &AccountKeys,
 ) -> Result<ParsedInstructionEnum, ParseInstructionError> {
     let bpf_loader_instruction: LoaderInstruction = deserialize(&instruction.data)
         .map_err(|_| ParseInstructionError::InstructionNotParsable(ParsableProgram::BpfLoader))?;
@@ -41,7 +41,7 @@ pub fn parse_bpf_loader(
 
 pub fn parse_bpf_upgradeable_loader(
     instruction: &CompiledInstruction,
-    account_keys: &[Pubkey],
+    account_keys: &AccountKeys,
 ) -> Result<ParsedInstructionEnum, ParseInstructionError> {
     let bpf_upgradeable_loader_instruction: UpgradeableLoaderInstruction =
         deserialize(&instruction.data).map_err(|_| {
@@ -159,7 +159,10 @@ mod test {
     use {
         super::*,
         serde_json::Value,
-        solana_sdk::{message::Message, pubkey},
+        solana_sdk::{
+            message::Message,
+            pubkey::{self, Pubkey},
+        },
     };
 
     #[test]
@@ -180,7 +183,11 @@ mod test {
         );
         let message = Message::new(&[instruction], Some(&fee_payer));
         assert_eq!(
-            parse_bpf_loader(&message.instructions[0], &account_keys).unwrap(),
+            parse_bpf_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&account_keys, None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "write".to_string(),
                 info: json!({
@@ -190,12 +197,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_loader(&message.instructions[0], &missing_account_keys).is_err());
+        assert!(parse_bpf_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&missing_account_keys, None)
+        )
+        .is_err());
 
         let instruction = solana_sdk::loader_instruction::finalize(&account_pubkey, &program_id);
         let message = Message::new(&[instruction], Some(&fee_payer));
         assert_eq!(
-            parse_bpf_loader(&message.instructions[0], &account_keys).unwrap(),
+            parse_bpf_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&account_keys, None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "finalize".to_string(),
                 info: json!({
@@ -203,21 +218,33 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_loader(&message.instructions[0], &missing_account_keys).is_err());
+        assert!(parse_bpf_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&missing_account_keys, None)
+        )
+        .is_err());
 
         let bad_compiled_instruction = CompiledInstruction {
             program_id_index: 3,
             accounts: vec![1, 2],
             data: vec![2, 0, 0, 0], // LoaderInstruction enum only has 2 variants
         };
-        assert!(parse_bpf_loader(&bad_compiled_instruction, &account_keys).is_err());
+        assert!(parse_bpf_loader(
+            &bad_compiled_instruction,
+            &AccountKeys::new(&account_keys, None)
+        )
+        .is_err());
 
         let bad_compiled_instruction = CompiledInstruction {
             program_id_index: 3,
             accounts: vec![],
             data: vec![1, 0, 0, 0],
         };
-        assert!(parse_bpf_loader(&bad_compiled_instruction, &account_keys).is_err());
+        assert!(parse_bpf_loader(
+            &bad_compiled_instruction,
+            &AccountKeys::new(&account_keys, None)
+        )
+        .is_err());
     }
 
     #[test]
@@ -240,7 +267,11 @@ mod test {
         .unwrap();
         let message = Message::new(&instructions, None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[1], &keys[0..3]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[1],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "initializeBuffer".to_string(),
                 info: json!({
@@ -249,13 +280,21 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[1], &keys[0..2]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[1],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction =
             solana_sdk::bpf_loader_upgradeable::write(&keys[1], &keys[0], offset, bytes.clone());
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "write".to_string(),
                 info: json!({
@@ -266,7 +305,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
 
         let instructions = solana_sdk::bpf_loader_upgradeable::deploy_with_max_program_len(
             &keys[0],
@@ -279,7 +322,11 @@ mod test {
         .unwrap();
         let message = Message::new(&instructions, None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[1], &keys[0..8]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[1],
+                &AccountKeys::new(&keys[0..8], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "deployWithMaxDataLen".to_string(),
                 info: json!({
@@ -295,13 +342,21 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[1], &keys[0..7]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[1],
+            &AccountKeys::new(&keys[0..7], None)
+        )
+        .is_err());
 
         let instruction =
             solana_sdk::bpf_loader_upgradeable::upgrade(&keys[2], &keys[3], &keys[0], &keys[4]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..7]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..7], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "upgrade".to_string(),
                 info: json!({
@@ -315,13 +370,21 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..6]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..6], None)
+        )
+        .is_err());
 
         let instruction =
             solana_sdk::bpf_loader_upgradeable::set_buffer_authority(&keys[1], &keys[0], &keys[2]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setAuthority".to_string(),
                 info: json!({
@@ -331,7 +394,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
 
         let instruction = solana_sdk::bpf_loader_upgradeable::set_upgrade_authority(
             &keys[1],
@@ -340,7 +407,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setAuthority".to_string(),
                 info: json!({
@@ -350,13 +421,21 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
 
         let instruction =
             solana_sdk::bpf_loader_upgradeable::set_upgrade_authority(&keys[1], &keys[0], None);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setAuthority".to_string(),
                 info: json!({
@@ -366,12 +445,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
 
         let instruction = solana_sdk::bpf_loader_upgradeable::close(&keys[0], &keys[1], &keys[2]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_bpf_upgradeable_loader(&message.instructions[0], &keys[..3]).unwrap(),
+            parse_bpf_upgradeable_loader(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "close".to_string(),
                 info: json!({
@@ -381,6 +468,10 @@ mod test {
                 }),
             }
         );
-        assert!(parse_bpf_upgradeable_loader(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_bpf_upgradeable_loader(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
     }
 }
