@@ -5,13 +5,14 @@ use {
     bincode::deserialize,
     serde_json::{json, Map},
     solana_sdk::{
-        instruction::CompiledInstruction, pubkey::Pubkey, stake::instruction::StakeInstruction,
+        instruction::CompiledInstruction, message::AccountKeys,
+        stake::instruction::StakeInstruction,
     },
 };
 
 pub fn parse_stake(
     instruction: &CompiledInstruction,
-    account_keys: &[Pubkey],
+    account_keys: &AccountKeys,
 ) -> Result<ParsedInstructionEnum, ParseInstructionError> {
     let stake_instruction: StakeInstruction = deserialize(&instruction.data)
         .map_err(|_| ParseInstructionError::InstructionNotParsable(ParsableProgram::Stake))?;
@@ -312,7 +313,11 @@ mod test {
             instruction::create_account(&keys[0], &keys[1], &authorized, &lockup, lamports);
         let message = Message::new(&instructions, None);
         assert_eq!(
-            parse_stake(&message.instructions[1], &keys[0..3]).unwrap(),
+            parse_stake(
+                &message.instructions[1],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "initialize".to_string(),
                 info: json!({
@@ -330,13 +335,21 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[1], &keys[0..2]).is_err());
+        assert!(parse_stake(
+            &message.instructions[1],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction =
             instruction::authorize(&keys[1], &keys[0], &keys[3], StakeAuthorize::Staker, None);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorize".to_string(),
                 info: json!({
@@ -348,7 +361,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction = instruction::authorize(
             &keys[2],
@@ -359,7 +376,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorize".to_string(),
                 info: json!({
@@ -372,12 +393,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction = instruction::delegate_stake(&keys[1], &keys[0], &keys[2]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..6]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..6], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "delegate".to_string(),
                 info: json!({
@@ -390,7 +419,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..5]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..5], None)
+        )
+        .is_err());
 
         // This looks wrong, but in an actual compiled instruction, the order is:
         //  * split account (signer, allocate + assign first)
@@ -399,7 +432,11 @@ mod test {
         let instructions = instruction::split(&keys[2], &keys[1], lamports, &keys[0]);
         let message = Message::new(&instructions, None);
         assert_eq!(
-            parse_stake(&message.instructions[2], &keys[0..3]).unwrap(),
+            parse_stake(
+                &message.instructions[2],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "split".to_string(),
                 info: json!({
@@ -410,12 +447,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[2], &keys[0..2]).is_err());
+        assert!(parse_stake(
+            &message.instructions[2],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction = instruction::withdraw(&keys[1], &keys[0], &keys[2], lamports, None);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..5], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "withdraw".to_string(),
                 info: json!({
@@ -432,7 +477,11 @@ mod test {
             instruction::withdraw(&keys[2], &keys[0], &keys[3], lamports, Some(&keys[1]));
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..6]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..6], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "withdraw".to_string(),
                 info: json!({
@@ -446,12 +495,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..4], None)
+        )
+        .is_err());
 
         let instruction = instruction::deactivate_stake(&keys[1], &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "deactivate".to_string(),
                 info: json!({
@@ -461,12 +518,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instructions = instruction::merge(&keys[1], &keys[0], &keys[2]);
         let message = Message::new(&instructions, None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..5], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "merge".to_string(),
                 info: json!({
@@ -478,7 +543,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..4], None)
+        )
+        .is_err());
 
         let seed = "test_seed";
         let instruction = instruction::authorize_with_seed(
@@ -492,7 +561,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeWithSeed".to_string(),
                 info: json!({
@@ -506,7 +579,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction = instruction::authorize_with_seed(
             &keys[2],
@@ -519,7 +596,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeWithSeed".to_string(),
                 info: json!({
@@ -534,7 +615,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..3]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
     }
 
     #[test]
@@ -556,7 +641,11 @@ mod test {
         let instruction = instruction::set_lockup(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setLockup".to_string(),
                 info: json!({
@@ -577,7 +666,11 @@ mod test {
         let instruction = instruction::set_lockup(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setLockup".to_string(),
                 info: json!({
@@ -599,7 +692,11 @@ mod test {
         let instruction = instruction::set_lockup(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setLockup".to_string(),
                 info: json!({
@@ -614,7 +711,11 @@ mod test {
             }
         );
 
-        assert!(parse_stake(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
 
         let lockup = LockupArgs {
             unix_timestamp: Some(unix_timestamp),
@@ -624,7 +725,11 @@ mod test {
         let instruction = instruction::set_lockup_checked(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setLockupChecked".to_string(),
                 info: json!({
@@ -645,7 +750,11 @@ mod test {
         let instruction = instruction::set_lockup_checked(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setLockupChecked".to_string(),
                 info: json!({
@@ -658,7 +767,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
 
         let lockup = LockupArgs {
             unix_timestamp: Some(unix_timestamp),
@@ -668,7 +781,11 @@ mod test {
         let instruction = instruction::set_lockup_checked(&keys[2], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "setLockupChecked".to_string(),
                 info: json!({
@@ -682,7 +799,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
     }
 
     #[test]
@@ -703,7 +824,11 @@ mod test {
             instruction::create_account_checked(&keys[0], &keys[1], &authorized, lamports);
         let message = Message::new(&instructions, None);
         assert_eq!(
-            parse_stake(&message.instructions[1], &keys[0..4]).unwrap(),
+            parse_stake(
+                &message.instructions[1],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "initializeChecked".to_string(),
                 info: json!({
@@ -714,7 +839,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[1], &keys[0..3]).is_err());
+        assert!(parse_stake(
+            &message.instructions[1],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
 
         let instruction = instruction::authorize_checked(
             &keys[2],
@@ -725,7 +854,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeChecked".to_string(),
                 info: json!({
@@ -737,7 +870,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..3]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
 
         let instruction = instruction::authorize_checked(
             &keys[3],
@@ -748,7 +885,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..5], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeChecked".to_string(),
                 info: json!({
@@ -761,7 +902,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..4], None)
+        )
+        .is_err());
 
         let seed = "test_seed";
         let instruction = instruction::authorize_checked_with_seed(
@@ -775,7 +920,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeCheckedWithSeed".to_string(),
                 info: json!({
@@ -789,7 +938,11 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..3]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
 
         let instruction = instruction::authorize_checked_with_seed(
             &keys[3],
@@ -802,7 +955,11 @@ mod test {
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
+            parse_stake(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..5], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeCheckedWithSeed".to_string(),
                 info: json!({
@@ -817,6 +974,10 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
+        assert!(parse_stake(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..4], None)
+        )
+        .is_err());
     }
 }
