@@ -4,13 +4,13 @@ use {
     },
     bincode::deserialize,
     serde_json::json,
-    solana_sdk::{instruction::CompiledInstruction, pubkey::Pubkey},
+    solana_sdk::{instruction::CompiledInstruction, message::AccountKeys},
     solana_vote_program::vote_instruction::VoteInstruction,
 };
 
 pub fn parse_vote(
     instruction: &CompiledInstruction,
-    account_keys: &[Pubkey],
+    account_keys: &AccountKeys,
 ) -> Result<ParsedInstructionEnum, ParseInstructionError> {
     let vote_instruction: VoteInstruction = deserialize(&instruction.data)
         .map_err(|_| ParseInstructionError::InstructionNotParsable(ParsableProgram::Vote))?;
@@ -70,6 +70,45 @@ pub fn parse_vote(
                 }),
             })
         }
+        VoteInstruction::UpdateVoteState(vote_state_update) => {
+            check_num_vote_accounts(&instruction.accounts, 4)?;
+            let vote_state_update = json!({
+                "lockouts": vote_state_update.lockouts,
+                "root": vote_state_update.root,
+                "hash": vote_state_update.hash.to_string(),
+                "timestamp": vote_state_update.timestamp,
+            });
+            Ok(ParsedInstructionEnum {
+                instruction_type: "updatevotestate".to_string(),
+                info: json!({
+                    "voteAccount": account_keys[instruction.accounts[0] as usize].to_string(),
+                    "slotHashesSysvar": account_keys[instruction.accounts[1] as usize].to_string(),
+                    "clockSysvar": account_keys[instruction.accounts[2] as usize].to_string(),
+                    "voteAuthority": account_keys[instruction.accounts[3] as usize].to_string(),
+                    "voteStateUpdate": vote_state_update,
+                }),
+            })
+        }
+        VoteInstruction::UpdateVoteStateSwitch(vote_state_update, hash) => {
+            check_num_vote_accounts(&instruction.accounts, 4)?;
+            let vote_state_update = json!({
+                "lockouts": vote_state_update.lockouts,
+                "root": vote_state_update.root,
+                "hash": vote_state_update.hash.to_string(),
+                "timestamp": vote_state_update.timestamp,
+            });
+            Ok(ParsedInstructionEnum {
+                instruction_type: "updatevotestateswitch".to_string(),
+                info: json!({
+                    "voteAccount": account_keys[instruction.accounts[0] as usize].to_string(),
+                    "slotHashesSysvar": account_keys[instruction.accounts[1] as usize].to_string(),
+                    "clockSysvar": account_keys[instruction.accounts[2] as usize].to_string(),
+                    "voteAuthority": account_keys[instruction.accounts[3] as usize].to_string(),
+                    "voteStateUpdate": vote_state_update,
+                    "hash": hash.to_string(),
+                }),
+            })
+        }
         VoteInstruction::Withdraw(lamports) => {
             check_num_vote_accounts(&instruction.accounts, 3)?;
             Ok(ParsedInstructionEnum {
@@ -188,7 +227,11 @@ mod test {
         );
         let message = Message::new(&instructions, None);
         assert_eq!(
-            parse_vote(&message.instructions[1], &keys[0..5]).unwrap(),
+            parse_vote(
+                &message.instructions[1],
+                &AccountKeys::new(&keys[0..5], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "initialize".to_string(),
                 info: json!({
@@ -202,13 +245,21 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[1], &keys[0..3]).is_err());
+        assert!(parse_vote(
+            &message.instructions[1],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
 
         let authority_type = VoteAuthorize::Voter;
         let instruction = vote_instruction::authorize(&keys[1], &keys[0], &keys[3], authority_type);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_vote(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_vote(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorize".to_string(),
                 info: json!({
@@ -220,12 +271,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_vote(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction = vote_instruction::vote(&keys[1], &keys[0], vote.clone());
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_vote(&message.instructions[0], &keys[0..4]).unwrap(),
+            parse_vote(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "vote".to_string(),
                 info: json!({
@@ -241,12 +300,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[0], &keys[0..3]).is_err());
+        assert!(parse_vote(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
 
         let instruction = vote_instruction::withdraw(&keys[1], &keys[0], lamports, &keys[2]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_vote(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_vote(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "withdraw".to_string(),
                 info: json!({
@@ -257,12 +324,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_vote(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction = vote_instruction::update_validator_identity(&keys[2], &keys[1], &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_vote(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_vote(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..3], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "updateValidatorIdentity".to_string(),
                 info: json!({
@@ -272,12 +347,20 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[0], &keys[0..2]).is_err());
+        assert!(parse_vote(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..2], None)
+        )
+        .is_err());
 
         let instruction = vote_instruction::update_commission(&keys[1], &keys[0], commission);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_vote(&message.instructions[0], &keys[0..2]).unwrap(),
+            parse_vote(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..2], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "updateCommission".to_string(),
                 info: json!({
@@ -287,13 +370,21 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_vote(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..1], None)
+        )
+        .is_err());
 
         let proof_hash = Hash::new_from_array([2; 32]);
         let instruction = vote_instruction::vote_switch(&keys[1], &keys[0], vote, proof_hash);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_vote(&message.instructions[0], &keys[0..4]).unwrap(),
+            parse_vote(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "voteSwitch".to_string(),
                 info: json!({
@@ -310,14 +401,22 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[0], &keys[0..3]).is_err());
+        assert!(parse_vote(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
 
         let authority_type = VoteAuthorize::Voter;
         let instruction =
             vote_instruction::authorize_checked(&keys[1], &keys[0], &keys[3], authority_type);
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_vote(&message.instructions[0], &keys[0..4]).unwrap(),
+            parse_vote(
+                &message.instructions[0],
+                &AccountKeys::new(&keys[0..4], None)
+            )
+            .unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeChecked".to_string(),
                 info: json!({
@@ -329,6 +428,10 @@ mod test {
                 }),
             }
         );
-        assert!(parse_vote(&message.instructions[0], &keys[0..3]).is_err());
+        assert!(parse_vote(
+            &message.instructions[0],
+            &AccountKeys::new(&keys[0..3], None)
+        )
+        .is_err());
     }
 }
